/*
 *  CArray.h
 *  ICExtensionMapping
 *
 *  Created by ostra on 2005.07.18.
 *  Copyright 2005 __MyCompanyName__. All rights reserved.
 *
 */
#ifndef __CARRAY_H__
#define __CARRAY_H__

#import <Cocoa/Cocoa.h>
#import <Carbon/Carbon.h>
#import "CString.h"
#import "CFile.h"

//
// 配列
//
class CArray {
protected:
	CFMutableArrayRef m_ref;
	void init();
	void deleteRef();
public:
	CArray();
	~CArray();
	CArray(CFArrayRef theArray);
	CArray(NSArray *theArray);
	Boolean isNull() {
		return m_ref == NULL;
	}
	operator NSMutableArray *() const {
		return (NSMutableArray *)m_ref;
	}
	void operator=(CFArrayRef theArray) {
		copyFromArray(theArray);
	}
	void copyFromArray(CFArrayRef theArray);
	void appendValue(const void *value) {
		CFArrayAppendValue(m_ref, value);
	}
	int getCount() {
		return (int)CFArrayGetCount(m_ref);
	}
	const void *operator[](int idx) {
		return getValueAtIndex(idx);
	}
	void add(const void *value) {
		appendValue(value);
	}
	const void *getValueAtIndex(int idx) {
		return CFArrayGetValueAtIndex(m_ref, (CFIndex)idx);
	}
	void removeAllValues();
	void removeDeleteAllValues();
	const void *removeValueAtIndex(int idx);
	void sortValues(CFComparatorFunction comparator, void *context);
	int getCountOfValue(const void *value);
	int getOrder(const void *value);
	virtual void deleteItem(const void *item) {
	}
};

//
// 文字列の配列
//
class CStringArray : public CArray {
public:
	CString *operator[](int idx) {
		return (CString *)getValueAtIndex(idx);
	}
	virtual void deleteItem(const void *item) {
		delete (CString *)item;
	}
	static CFComparisonResult comparator(const void *val1, const void *val2, void *context);
	void sort() {
		sortValues(comparator, NULL);
	}
};

//
// メモリブロック
//
class CData {
protected:
	CFMutableDataRef m_ref;
	void init();
	void deleteRef();
public:
	CData();
	~CData();
	CData(int length);
	CData(const void *bytes, int length);
	CData(CResource res);
	operator void *() const {
		return getBytePtr();
	}
	operator CFMutableDataRef() const {
		return m_ref;
	}
	operator CFDataRef() const {
		return m_ref;
	}
	void setLength(int length) {
		CFDataSetLength(m_ref, (CFIndex)length);
	}
	void create(int capacity = 0) {
		m_ref = CFDataCreateMutable(kCFAllocatorDefault, (CFIndex)capacity);
	}
	void copyFromBytes(const void *bytes, int length);
	void *getBytePtr() const {
		return (void *)CFDataGetMutableBytePtr(m_ref);
	}
	int getLength() {
		return CFDataGetLength(m_ref);
	}
	void getBytes(int offset, int length, void *buffer);
};

//
// メモリブロックからのバイナリ読み込み
//
class CBinaryReader {
	CFDataRef m_ref;
	int m_position;
public:
	CBinaryReader(const CFDataRef data);
	int getLength() {
		return CFDataGetLength(m_ref);
	}
	void resetPosition() {
		m_position = 0;
	}
	void getBytes(int offset, int length, void *buffer);
	int readInt();
	short readShort();
	char readChar();
};

//
// ディクショナリ（マップ）
//
class CDictionary {
protected:
	CFMutableDictionaryRef m_ref;
	void init();
	void deleteRef();
public:
	CDictionary();
	~CDictionary();
	CDictionary(CFDictionaryRef theDict);
	operator CFMutableDictionaryRef() const {
		return m_ref;
	}
	int getCount() {
		return (int)CFDictionaryGetCount(m_ref);
	}
	const void *getValue(CFStringRef key) {
		return CFDictionaryGetValue(m_ref, (const void *)key);
	}
	void copyFromDictionary(CFDictionaryRef theDict) {
		m_ref = CFDictionaryCreateMutableCopy(kCFAllocatorDefault, 0, theDict);
	}
};

#endif
