/*
 *  CArray.mm
 *  ICExtensionMapping
 *
 *  Created by ostra on 2005.08.13.
 *  Copyright 2005 __MyCompanyName__. All rights reserved.
 *
 */

#import "CCollection.h"
#import "CString.h"
#import "CFile.h"

void CArray::init() {
	m_ref = NULL;
}

void CArray::deleteRef() {
	if (m_ref != NULL)
		CFRelease(m_ref);
	m_ref = NULL;
}

CArray::CArray() {
	init();
	m_ref = CFArrayCreateMutable(kCFAllocatorDefault, 0, NULL);
}

CArray::~CArray() {
	removeDeleteAllValues();
	deleteRef();
}

CArray::CArray(CFArrayRef theArray) {
	init();
	copyFromArray(theArray);
}

CArray::CArray(NSArray *theArray) {
	init();
	copyFromArray((CFArrayRef)theArray);
}

void CArray::copyFromArray(CFArrayRef theArray) {
	if (theArray != NULL)
		m_ref = CFArrayCreateMutableCopy(kCFAllocatorDefault, 0, theArray);
}

const void *CArray::removeValueAtIndex(int idx) {
	const void *item = getValueAtIndex(idx);
	CFArrayRemoveValueAtIndex(m_ref, (CFIndex)idx);
	return item;
}

void CArray::removeDeleteAllValues() {
	if (m_ref != NULL) {
		for (int i = 0; i < getCount(); i++)
			deleteItem(getValueAtIndex(i));
		removeAllValues();
	}
}

void CArray::removeAllValues() {
	if (m_ref != NULL)
		CFArrayRemoveAllValues(m_ref);
}

void CArray::sortValues(CFComparatorFunction comparator, void *context) {
	CFRange range = {0, getCount()};
	CFArraySortValues(m_ref, range, comparator, context);
}

int CArray::getCountOfValue(const void *value) {
	CFRange range = {0, getCount()};
	return (int)CFArrayGetCountOfValue(m_ref, range, value);
}

int CArray::getOrder(const void *value) {
	for (int i = 0; i < getCount(); i++) {
		if (getValueAtIndex(i) == value)
			return i;
	}
	return -1;
}

CFComparisonResult CStringArray::comparator(const void *val1, const void *val2, void *context) {
	CString *s1 = (CString *)val1;
	CString *s2 = (CString *)val2;
	return (CFComparisonResult)(*s1).compare(*s2);
}

void CData::init() {
	m_ref = NULL;
}

void CData::deleteRef() {
	if (m_ref != NULL)
		CFRelease(m_ref);
	m_ref = NULL;
}

CData::CData() {
	init();
}

CData::~CData() {
	deleteRef();
}

CData::CData(int length) {
	init();
	create();
	setLength(length);
}

CData::CData(const void *bytes, int length) {
	init();
	copyFromBytes(bytes, length);
}

CData::CData(CResource res) {
	init();
	copyFromBytes((const UInt8 *)res.lock(), res.getSize());
	res.unlock();
}

void CData::copyFromBytes(const void *bytes, int length) {
	CFDataRef dt = CFDataCreateWithBytesNoCopy(kCFAllocatorDefault, (const UInt8 *)bytes, (CFIndex)length, kCFAllocatorNull);
	m_ref = CFDataCreateMutableCopy(kCFAllocatorDefault, 0, dt);
}

void CData::getBytes(int offset, int length, void *buffer) {
	CFRange range = {offset, length};
	CFDataGetBytes(m_ref, range, (UInt8 *)buffer);
}

CBinaryReader::CBinaryReader(const CFDataRef data) {
	m_ref = data;
	m_position = 0;
}

void CBinaryReader::getBytes(int offset, int length, void *buffer) {
	CFRange range = {offset, length};
	CFDataGetBytes(m_ref, range, (UInt8 *)buffer);
}

int CBinaryReader::readInt() {
	int result;
	getBytes(m_position, sizeof(result), &result);
	m_position += sizeof(result);
	return ntohl(result);
}

short CBinaryReader::readShort() {
	short result;
	getBytes(m_position, sizeof(result), &result);
	m_position += sizeof(result);
	return ntohs(result);
}

char CBinaryReader::readChar() {
	char result;
	getBytes(m_position, sizeof(result), &result);
	m_position += sizeof(result);
	return result;
}

void CDictionary::init() {
	m_ref = NULL;
}

void CDictionary::deleteRef() {
	if (m_ref != NULL)
		CFRelease(m_ref);
	m_ref = NULL;
}

CDictionary::CDictionary() {
	init();
	m_ref = CFDictionaryCreateMutable(kCFAllocatorDefault, 0, NULL, NULL);
}

CDictionary::~CDictionary() {
	deleteRef();
}

CDictionary::CDictionary(CFDictionaryRef theDict) {
	init();
	copyFromDictionary(theDict);
}

