/*
 *  CFile.h
 *  ICExtensionMapping
 *
 *  Created by ostra on 2005.08.09.
 *  Copyright 2005 __MyCompanyName__. All rights reserved.
 *
 */

#ifndef __CFILE_H__
#define __CFILE_H__

#import <Cocoa/Cocoa.h>
#import <Carbon/Carbon.h>
#import "CString.h"

class CURL {
	CFURLRef m_ref;
	void init();
	void deleteRef();
public:
	CURL();
	CURL(CFURLRef relativeURL);
	CURL(CFStringRef filePath, Boolean isDirectory = false);
	void operator=(CFURLRef relativeURL) {
		copyFromURL(relativeURL);
	}
	void operator=(CFStringRef filePath) {
		copyFromString(filePath);
	}
	operator CFURLRef() const {
		return m_ref;
	}
	void copyFromURL(CFURLRef relativeURL);
	void copyFromString(CFStringRef filePath, Boolean isDirectory = false);
};

class CFSRef {
	FSRef m_ref;
public:
	CFSRef() {
	}
	CFSRef(const CString& path) {
		makeByPath(path);
	}
	CFSRef(const CFURLRef url) {
		copyFromURL(url);
	}
	~CFSRef() {
	}
	const FSRef* getRefPtr() {
		return &m_ref;
	}
	operator const FSRef *() const {
		return &m_ref;
	}
	operator FSSpec() {
		return getFSSpec();
	}
	Boolean copyFromURL(const CFURLRef url) {
		return CFURLGetFSRef(url, &m_ref);
	}
	OSStatus makeByPath(const CString& path);
	OSStatus getCatalogInfo(FSCatalogInfo *catalogInfo, HFSUniStr255 *outName = NULL, FSSpec *fsSpec = NULL, FSRef *parentRef = NULL);
	FSSpec getFSSpec();
};

class CReadStream {
	CFReadStreamRef m_ref;
	void init();
	void deleteRef();
public:
	CReadStream(const CURL& url) {
		init();
		makeByURL(url);
	}
	operator CFReadStreamRef() const {
		return m_ref;
	}
	Boolean open() {
		return CFReadStreamOpen(m_ref);
	}
	void close() {
		CFReadStreamClose(m_ref);
	}
	int read(void *buffer, int bufferLength) {
		return CFReadStreamRead(m_ref, (UInt8 *)buffer, (CFIndex)bufferLength);
	}
	void makeByURL(const CURL& url) {
		m_ref = CFReadStreamCreateWithFile(kCFAllocatorDefault, url);
	}
};

class CResource {
protected:
	Handle m_h;
public:
	void setHandle(Handle h) {
		m_h = h;
	}
	Boolean isNull() {
		return m_h == NULL;
	}
	operator Handle() {
		return m_h;
	}
	int getSize() {
		return GetHandleSize(m_h);
	}
	void *lock();
	void unlock();
};

class CResourceFile {
protected:
	short m_ref;
	void init();
public:
	Boolean isNull() {
		return m_ref == 0;
	}
	Boolean open(const FSSpec& spec, SignedByte permission = fsRdPerm);
	void close();
	CResource CResourceFile::getResource(OSType sig, short id);
};

#endif
