/*
 *  CArray.h
 *  ICExtensionMapping
 *
 *  Created by ostra on 2005.07.18.
 *  Copyright 2005 __MyCompanyName__. All rights reserved.
 *
 */
#ifndef __CSTRING_H__
#define __CSTRING_H__

#import <Cocoa/Cocoa.h>
#import <Carbon/Carbon.h>

class CString {
	CFStringRef m_ref;
	void init();
	void deleteRef();
public:
	CString();
	~CString();
	CString(const CString &cs);
	CString(const char *cstr);
	CString(ConstStr255Param pStr);
	CString(CFStringRef theString);
	CString(NSString *theString);
	CString(OSType sig);
	CString(CFURLRef anURL);
	void operator=(const CString &cs) {
		copyFromString(cs);
	}
	void operator=(const char *cstr) {
		copyFromCSTR(cstr);
	}
	void operator=(ConstStr255Param pStr) {
		copyFromPSTR(pStr);
	}
	void operator=(CFStringRef theString) {
		copyFromString(theString);
	}
	void operator=(NSString *theString) {
		copyFromString((CFStringRef)theString);
	}
	void operator=(OSType sig) {
		copyFromOSType(sig);
	}
	void operator=(CFURLRef anURL) {
		copyFromURL(anURL);
	}
	operator NSString *() const {
		return (NSString *)m_ref;
	}
	operator CFStringRef() const {
		return m_ref;
	}
	operator const char *() const {
		return CFStringGetCStringPtr(m_ref, kCFStringEncodingUTF8);
	}
	operator const UniChar *() const {
		return CFStringGetCharactersPtr(m_ref);
	}
	operator OSType() {
		return getOSType();
	}
	int compare(const CString &s, CFOptionFlags option = 0) {
		return (int)CFStringCompare(m_ref, s.m_ref, option);
	}
	int compareNoCase(const CString &s) {
		return compare(s, kCFCompareCaseInsensitive);
	}
	Boolean equals(const CString &s) {
		return compare(s) == 0;
	}
	Boolean operator==(const CString &s) {
		return equals(s);
	}
	Boolean operator!=(const CString &s) {
		return !equals(s);
	}
	int getLength() const {
		return (int)CFStringGetLength(m_ref);
	}
	CString operator+(const CString &s) {
		return append(s);
	}
	CString operator+(const char *cstr) {
		return append(cstr);
	}
	Boolean hasSuffix(const CString& s) {
		return CFStringHasSuffix(m_ref, s);
	}
	CString append(const CString& s);
	void copyFromCSTR(const char *cstr, CFStringEncoding encoding = kCFStringEncodingUTF8);
	void copyFromPSTR(ConstStr255Param pStr, CFStringEncoding encoding = kCFStringEncodingUTF8);
	void copyFromString(CFStringRef theString);
	void copyFromOSType(OSType sig);
	void copyFromURL(CFURLRef anURL);
	void copyToPSTR(StringPtr buffer, int bufferSize = 255, CFStringEncoding encoding = kCFStringEncodingUTF8) const;
	void copyToCSTR(char *buffer, int bufferSize, CFStringEncoding encoding = kCFStringEncodingUTF8) const;
	void copyToUSTR(UniChar *buffer, int bufferSize) const;
	OSType getOSType();
};

#endif
