/*
 *  CString.mm
 *  ICExtensionMapping
 *
 *  Created by ostra on 2005.07.20.
 *  Copyright 2005 __MyCompanyName__. All rights reserved.
 *
 */

#import <Cocoa/Cocoa.h>
#import <Carbon/Carbon.h>
#import "CString.h"

static CFStringRef emptyString = CFSTR("");

void CString::init() {
	m_ref = emptyString;
}

void CString::deleteRef() {
	if (m_ref != NULL && m_ref != emptyString)
		CFRelease(m_ref);
	m_ref = emptyString;
}


CString::CString() {
	init();
}

CString::~CString() {
	deleteRef();
}

CString::CString(const CString &cs) {
	init();
	copyFromString(cs);
}

CString::CString(const char *cstr) {
	init();
	copyFromCSTR(cstr);
}

CString::CString(ConstStr255Param pStr) {
	init();
	copyFromPSTR(pStr);
}

CString::CString(CFStringRef theString) {
	init();
	copyFromString(theString);
}

CString::CString(NSString *theString) {
	init();
	copyFromString((CFStringRef)theString);
}

CString::CString(OSType sig) {
	init();
	copyFromOSType(sig);
}

CString::CString(CFURLRef anURL) {
	init();
	copyFromURL(anURL);
}

CString CString::append(const CString& s) {
	CFMutableStringRef m = CFStringCreateMutableCopy(kCFAllocatorDefault, 0, m_ref);
	CFStringAppend(m, s);
	return CString(m);
	//copyFromString(m);
}

void CString::copyFromCSTR(const char *cstr, CFStringEncoding encoding) {
	deleteRef();
	if (cstr != NULL)
		m_ref = CFStringCreateWithCString(kCFAllocatorDefault, cstr, encoding);
}

void CString::copyFromPSTR(ConstStr255Param pStr, CFStringEncoding encoding) {
	deleteRef();
	m_ref = CFStringCreateWithPascalString(kCFAllocatorDefault, pStr, encoding);
}

void CString::copyFromString(CFStringRef theString) {
	deleteRef();
	if (theString != NULL)
		m_ref = CFStringCreateCopy(kCFAllocatorDefault, theString);
}

void CString::copyFromURL(CFURLRef anURL) {
	copyFromString(CFURLGetString(anURL));
}

void CString::copyFromOSType(OSType sig) {
	char bf[5];
	int shift = 24;
	for (int i = 0; i < 4; i++) {
		bf[i] = ((sig >> shift) & 0x000000FF);
		shift -= 8;
	}
	bf[4] = '\0';
	*this = bf;
}

void CString::copyToPSTR(StringPtr buffer, int bufferSize, CFStringEncoding encoding) const {
	CFStringGetPascalString(m_ref, buffer, (CFIndex)bufferSize, encoding);
}

void CString::copyToCSTR(char *buffer, int bufferSize, CFStringEncoding encoding) const {
	CFStringGetCString(m_ref, buffer, (CFIndex)bufferSize, encoding);
}

void CString::copyToUSTR(UniChar *buffer, int bufferSize) const {
	CFRange range = {0, bufferSize};
	CFStringGetCharacters(m_ref, range, buffer);
}

OSType CString::getOSType() {
	OSType sig = 0;
	if (getLength() == 0)
		return sig;
	char p[16];
	copyToCSTR(p, 15);
	char buffer[4] = {0x20, 0x20, 0x20, 0x20};
	int i;
	int count = strlen(p);
	for (i = 0; i < 4 && i < count; i++) {
		buffer[i] = p[i];
	}
	int shift = 24;
	for (i = 0; i < 4; i++) {
		unsigned long x = (buffer[i] & 0x000000FF);
		x = (x == 0 ? 0x20 : x);
		x = (x << shift);
		sig |= x;
		shift -= 8;
	}
	return sig;
}

