/*
 *  TApplInfo.mm
 *  ICExtensionMapping
 *
 *  Created by ostra on 2005.08.13.
 *  Copyright 2005 __MyCompanyName__. All rights reserved.
 *
 */

#import <Cocoa/Cocoa.h>
#import <Carbon/Carbon.h>
#import "CClass.h"
#import "TApplInfo.h"

void TApplInfo::clear() {
	m_path = "";
	m_fdTypes.removeDeleteAllValues();
	m_creator = "";
}

void TApplInfo::load(CFStringRef path) {
	clear();
	m_path = path;

	CFSRef selectedFile(path);
	FSCatalogInfo applCatInfo;
	selectedFile.getCatalogInfo(&applCatInfo);

	// ディレクトリ
	if (applCatInfo.nodeFlags & kFSNodeIsDirectoryMask) {
		if ([[NSWorkspace sharedWorkspace] isFilePackageAtPath:m_path]) {
			CURL infoPlistURL(m_path + "/Contents/info.plist");
			CFSRef infoPlistFile(infoPlistURL);
			FSCatalogInfo plistCatInfo;
			if (infoPlistFile.getCatalogInfo(&plistCatInfo) == noErr) {
				loadCocoa(infoPlistURL, (int)plistCatInfo.dataLogicalSize);
			}
		}
	}
	// リソースファイル
	else if (applCatInfo.rsrcLogicalSize > 0) {
		CResourceFile resFile;
		if (resFile.open(selectedFile)) {
			// 'plst'
			CResource plstRes = resFile.getResource('plst', 0);
			if (plstRes.isNull() == false) {
				CData buffer(plstRes);
				loadInfoPlistData(buffer);
			}
			// 'BNDL'
			else {
				CResource bndlRes = resFile.getResource('BNDL', 128);
				if (bndlRes.isNull() == false) {
					CData bndlData(bndlRes);
					loadBNDLData(resFile, bndlData);
				}
			}
			resFile.close();
		}
	}
	if (m_fdTypes.getCount() > 0)
		m_fdTypes.sort();
}

void TApplInfo::loadCocoa(CFURLRef infoPlistURL, int fileSize) {
	CData buffer(fileSize);
	CReadStream stream(infoPlistURL);
	if (stream.open()) {
		stream.read(buffer, fileSize);
		stream.close();
		loadInfoPlistData(buffer);
	}
}

void TApplInfo::loadInfoPlistData(CData& data) {
	CFPropertyListRef pl = CFPropertyListCreateFromXMLData(kCFAllocatorDefault, data, 0, NULL);
	CDictionary dic((CFDictionaryRef)pl);
	m_creator = (CFStringRef)dic.getValue(CString("CFBundleSignature"));
	CArray docTypes = (CFArrayRef)dic.getValue(CString("CFBundleDocumentTypes"));
	for (int i = 0; docTypes.isNull() == false && i < docTypes.getCount(); i++) {
		//CFTypeID kt = CFGetTypeID(docTypes[i]);
		CDictionary dtdic = (CFDictionaryRef)docTypes[i];
		CArray types = (CFArrayRef)dtdic.getValue(CString("CFBundleTypeOSTypes"));
		for (int j = 0; types.isNull() == false && j < types.getCount(); j++) {
			//CFTypeID kttt = CFGetTypeID(types[j]);
			//CString type = (CFStringRef)types[j];
			m_fdTypes.add(new CString((CFStringRef)types[j]));
			//int y = 0;
		}
	}
}

void TApplInfo::loadBNDLData(CResourceFile& resFile, CData& bndlData) {
	CBinaryReader bndlReader(bndlData);
	m_creator = (OSType)bndlReader.readInt();
	bndlReader.readInt();
	int type = bndlReader.readInt();
	if (type != 'FREF')
		return;
	int n = bndlReader.readShort();
	if (n == 0)
		return;
	for (int i = 0; i < n; i++) {
		bndlReader.readShort();
		short rsrcID = bndlReader.readShort();
		CResource frefRes = resFile.getResource('FREF', rsrcID);
		if (frefRes.isNull() == false) {
			CData frefData(frefRes);
			CBinaryReader frefReader(frefData);
			m_fdTypes.add(new CString((OSType)frefReader.readInt()));
		}
	}
}

