/*
 *  TEntry.mm
 *  ICExtensionMapping
 *
 *  Created by ostra on 2005.07.18.
 *  Copyright 2005 __MyCompanyName__. All rights reserved.
 *
 */

#include <Cocoa/Cocoa.h>
#include <Carbon/Carbon.h>
#include "TEntry.h"

TEntry::TEntry() {
	memset(&m_data, 0 , sizeof (m_data));
	m_data.totalLength = sizeof (m_data);
	m_data.fixedLength = kICMapFixedLength;
	m_dirty = false;
	m_indexNumber = -1;
	m_pos = -1;
}

TEntry *TEntry::clone() {
	TEntry *e = new TEntry;
	(*e).copyFrom(*this);
	return e;
}

void TEntry::copyFrom(const TEntry &e) {
	m_data = e.m_data;
	m_extension = e.m_extension;
	m_fileCreator = e.m_fileCreator;
	m_fileType = e.m_fileType;
	m_creatorAppName = e.m_creatorAppName;
	m_mimeType = e.m_mimeType;
	m_entryName = e.m_entryName;
	m_postCreator = e.m_postCreator;
	m_postAppName = e.m_postAppName;
}

Boolean TEntry::equals(const TEntry &e) {
	return (m_extension == e.m_extension &&
		m_fileCreator == e.m_fileCreator &&
		m_fileType == e.m_fileType &&
		m_creatorAppName == e.m_creatorAppName &&
		m_mimeType == e.m_mimeType &&
		m_entryName == e.m_entryName &&
		m_postCreator == e.m_postCreator &&
		m_postAppName == e.m_postAppName);
}

void TEntry::toData() {
	m_extension.copyToPSTR(m_data.extension);
	m_data.fileCreator = m_fileCreator;
	m_data.fileType = m_fileType;
	m_creatorAppName.copyToPSTR(m_data.creatorAppName);
	m_mimeType.copyToPSTR(m_data.MIMEType);
	m_entryName.copyToPSTR(m_data.entryName);
	m_data.postCreator = m_postCreator;
	m_postAppName.copyToPSTR(m_data.postAppName);
}

void TEntry::saveNewMapEntry(ICInstance ic, Handle mappingPrefHandle) {
	toData();
	ICAddMapEntry(ic, mappingPrefHandle, &m_data);
	m_dirty = false;
}

void TEntry::saveMapEntry(ICInstance ic, Handle mappingPrefHandle) {
	toData();
	ICSetMapEntry(ic, mappingPrefHandle, m_pos, &m_data);
	m_dirty = false;
}

Boolean TEntryMap::attachIC() {
	m_ic = NULL;
	m_mappingPrefHandle = NULL;
	Boolean good = TRUE;
	if (ICStart(&m_ic, 'ICEM') != noErr)
		good = FALSE;
	m_mappingPrefHandle = NewHandle(16);
	if (good && ICGetDefaultPref(m_ic, kICMapping, m_mappingPrefHandle) != noErr)
		good = FALSE;
	return good;
}

TEntryMap::~TEntryMap() {
	detachIC();
}

void TEntryMap::detachIC() {
	if (m_mappingPrefHandle != NULL) {
		DisposeHandle(m_mappingPrefHandle);
		m_mappingPrefHandle = NULL;
	}
	if (m_ic != NULL) {
		ICStop(m_ic);
		m_ic = NULL;
	}
}

Boolean TEntryMap::createEntryArray() {
	m_deleted.removeDeleteAllValues();
	removeDeleteAllValues();
	long count = 0;
	if (ICCountMapEntries(m_ic, m_mappingPrefHandle, &count) != noErr)
		return false;
	ICMapEntry entry;
	memset(&entry, 0 , sizeof (entry));
	entry.totalLength = sizeof (entry);
	entry.fixedLength = kICMapFixedLength;
	for (int i = 0; i < count; i++) {
		TEntry *e = new TEntry;
		(*e).m_indexNumber = i + 1;
		(*e).m_pos = 0;
		if (ICGetIndMapEntry(m_ic, m_mappingPrefHandle, (*e).m_indexNumber, &(*e).m_pos, &(*e).m_data) != noErr) {
			delete e;
			continue;
		}
		(*e).m_extension = (*e).m_data.extension;
		(*e).m_fileCreator = (*e).m_data.fileCreator;
		(*e).m_fileType = (*e).m_data.fileType;
		(*e).m_creatorAppName = (*e).m_data.creatorAppName;
		(*e).m_mimeType = (*e).m_data.MIMEType;
		(*e).m_entryName = (*e).m_data.entryName;
		(*e).m_postCreator = (*e).m_data.postCreator;
		(*e).m_postAppName = (*e).m_data.postAppName;
		add(e);
	}
	return true;
}

TEntry *TEntryMap::serarchResemble(const CString &extension, const CString &mimeType, const CString &fileCreator, const CString &fileType, TEntry *except) {
	int count = getCount();
	int i;
	for (i = 0; i < count; i++) {
		TEntry *e = (TEntry *)(*this)[i];
		if ((*e).m_extension == extension &&
			(*e).m_mimeType == mimeType &&
			(*e).m_fileCreator == fileCreator &&
			(*e).m_fileType == fileType) {
			if (except == NULL || except != e)
				return e;
		}
	}
	return NULL;
}

Boolean TEntryMap::hasDirty() {
	if (m_deleted.getCount() > 0)
		return true;
	int count = getCount();
	int i;
	for (i = 0; i < count; i++) {
		TEntry *e = (TEntry *)(*this)[i];
		if ((*e).m_dirty)
			return true;
	}
	return false;
}

Boolean TEntryMap::saveMapEntry() {
	if (hasDirty() == false)
		return false;

	int count = m_deleted.getCount();
	int i;
	for (i = 0; i < count; i++) {
		TEntry *e = (TEntry *)m_deleted[i];
		if ((*e).m_indexNumber >= 0) {
			ICDeleteMapEntry(m_ic, m_mappingPrefHandle, (*e).m_pos);
		}
	}
	m_deleted.removeDeleteAllValues();

	count = getCount();
	for (i = 0; i < count; i++) {
		TEntry *e = (TEntry *)(*this)[i];
		if ((*e).m_dirty == false)
			continue;

		if ((*e).m_indexNumber < 0) {
			(*e).saveNewMapEntry(m_ic, m_mappingPrefHandle);
		}
		else {
			(*e).saveMapEntry(m_ic, m_mappingPrefHandle);
		}
		(*e).m_dirty = false;
	}
	ICSetPrefHandle(m_ic, kICMapping, 0, m_mappingPrefHandle);
	return true;
}

void TEntryMap::onDelete(int i) {
	m_deleted.add(removeValueAtIndex(i));
}

CFComparisonResult TEntryMap::comparator(const void *val1, const void *val2, void *context) {
	TEntry *e1 = (TEntry *)val1;
	TEntry *e2 = (TEntry *)val2;
	TEntryMap::SortContext *sc = (TEntryMap::SortContext *)context;
	int asc = (*sc).m_ascending ? 1 : (-1);
	int n = 0;
	if ((*sc).m_field.equals("RS_EXTENSION"))
		n = (*e1).m_extension.compareNoCase((*e2).m_extension);
	else if ((*sc).m_field.equals("RS_CREATOR"))
		n = (*e1).m_fileCreator.compareNoCase((*e2).m_fileCreator);
	else if ((*sc).m_field.equals("RS_FILETYPE"))
		n = (*e1).m_fileType.compareNoCase((*e2).m_fileType);
	else if ((*sc).m_field.equals("RS_APPLICATION"))
		n = (*e1).m_creatorAppName.compareNoCase((*e2).m_creatorAppName);
	else if ((*sc).m_field.equals("RS_MIMETYPE"))
		n = (*e1).m_mimeType.compareNoCase((*e2).m_mimeType);

	if (n == 0)
		n = (*e1).m_extension.compareNoCase((*e2).m_extension);
	if (n == 0)
		n = (*e1).m_fileCreator.compareNoCase((*e2).m_fileCreator);
	if (n == 0)
		n = (*e1).m_fileType.compareNoCase((*e2).m_fileType);
	if (n == 0)
		n = (*e1).m_creatorAppName.compareNoCase((*e2).m_creatorAppName);
	if (n == 0)
		n = (*e1).m_mimeType.compareNoCase((*e2).m_mimeType);

	return (CFComparisonResult)(n * asc);
}

