#import "TMain.h"

@implementation TMain

#define CURRENT_PREF_VERSION	0x100

/*
 * nibŃCX^X
 */
- (void)awakeFromNib
{
	m_newEntry = NULL;
	m_array = new TEntryMap;
	if ((*m_array).attachIC() == false) {
		// Alert
		[[NSApplication sharedApplication] terminate:self];
	}
	(*m_array).createEntryArray();

	[mainWindow setDelegate:self];

	[fileMenu setDelegate:self];
	[commandMenu setDelegate:self];

	[tvEntry setDataSource:self];
	[tvEntry setDelegate:self];
	[tvEntry setDoubleAction:@selector(doubleClickTvEntryAction:)];

	[self loadSetting];

	(*m_array).m_sortContext.m_field = "RS_EXTENSION";
	(*m_array).sortValues(TEntryMap::comparator, &(*m_array).m_sortContext);
	[tvEntry reloadData];

	[[NSApplication sharedApplication] setDelegate:self];

	[self fixControl];
}

- (void)loadSetting {
	NSUserDefaults *ud = [NSUserDefaults standardUserDefaults];
	int version = [ud integerForKey:@"version"];
	if (version == 0) {
		return;
	}
	CString sortKey = [ud stringForKey:@"sortKey"];
	if (sortKey.getLength() > 0)
		(*m_array).m_sortContext.m_field = sortKey;
	CArray colums([tvEntry tableColumns]);
	int i;
	for (i = 0; i < colums.getCount(); i++) {
		NSTableColumn *column = (NSTableColumn *)colums[i];
		float width = [ud floatForKey:[column identifier]];
		if (width > 0.0) {
			[column setWidth:width];
		}
	}
}

- (void)saveSetting {
	NSUserDefaults *ud = [NSUserDefaults standardUserDefaults];
	[ud setInteger:CURRENT_PREF_VERSION forKey:@"version"];
	[ud setObject:(NSString *)(*m_array).m_sortContext.m_field forKey:@"sortKey"];
	CArray colums([tvEntry tableColumns]);
	int i;
	for (i = 0; i < colums.getCount(); i++) {
		NSTableColumn *column = (NSTableColumn *)colums[i];
		[ud setFloat:[column width] forKey:[column identifier]];
	}
	[ud synchronize];
}

/*
 * e[ur[R[obN
 */

- (int)numberOfRowsInTableView:(NSTableView *)tv {
	if (m_array == NULL)
		return 0;
	return (*m_array).getCount();
}

- (id)tableView:(NSTableView *)tv objectValueForTableColumn:(NSTableColumn *)column row:(int)row {
	if (m_array == NULL)
		return nil;
	if ((*m_array).getCount() <= row)
		return nil;
	TEntry *e = (TEntry *)(*m_array)[row];
	if ([[column identifier] isEqualToString: @"RS_EXTENSION"]) {
		return (*e).m_extension;
	}
	else if ([[column identifier] isEqualToString: @"RS_CREATOR"]) {
		return (*e).m_fileCreator;
	}
	else if ([[column identifier] isEqualToString: @"RS_FILETYPE"]) {
		return (*e).m_fileType;
	}
	else if ([[column identifier] isEqualToString: @"RS_APPLICATION"]) {
		return (*e).m_creatorAppName;
	}
	else if ([[column identifier] isEqualToString: @"RS_MIMETYPE"]) {
		return (*e).m_mimeType;
	}
	return nil;
}

- (void)tableView:(NSTableView *)tv didClickTableColumn:(NSTableColumn *)column {
/*
	NSSortDescriptor *sdp = [tableColumn sortDescriptorPrototype];
	if (sdp == nil)
		return;
	BOOL asc = [sdp ascending];
*/

	(*m_array).m_sortContext.m_field = [column identifier];
	(*m_array).sortValues(TEntryMap::comparator, &(*m_array).m_sortContext);
	[tvEntry reloadData];
}

- (void)tableViewSelectionDidChange:(NSNotification *)aNotification {
	[self fixControl];
}

- (void)fixControl {
	int n = [tvEntry selectedRow];
	if (n < 0) {
		[btnModify setEnabled:NO];
		[btnDelete setEnabled:NO];
	}
	else {
		[btnModify setEnabled:YES];
		[btnDelete setEnabled:YES];
	}
}

- (BOOL)validateMenuItem:(id <NSMenuItem>)menuItem {
	int n = [tvEntry selectedRow];
	Boolean canEdit = (n >= 0);
	Boolean dirty = (*m_array).hasDirty();
	if (menuItem == menuModify)
		return canEdit;
	if (menuItem == menuDelete)
		return canEdit;
	if (menuItem == menuSave)
		return dirty;
	if (menuItem == menuRevert)
		return dirty;
	return YES;
}

- (IBAction)menuModifyAction:(id)sender
{
	int n = [tvEntry selectedRow];
	if (n < 0)
		return;
	TEntry *e = (TEntry *)(*m_array)[n];
	[mappingDlg setup:e andArray:m_array];
//	if ([mappingDlg doModal] == NO)
//		return;
	[NSApp beginSheet:[mappingDlg window] modalForWindow:mainWindow modalDelegate:self didEndSelector:@selector(sheetDidEnd:returnCode:contextInfo:) contextInfo:nil];
}

- (IBAction)menuNewAction:(id)sender {
	m_newEntry = new TEntry;
	[mappingDlg setup:m_newEntry andArray:m_array];
	[NSApp beginSheet:[mappingDlg window] modalForWindow:mainWindow modalDelegate:self didEndSelector:@selector(sheetDidEnd:returnCode:contextInfo:) contextInfo:nil];

/*
	if ([mappingDlg doModal] == NO) {
		delete e;
		return;
	}
	(*m_array).add(e);
	(*m_array).sortValues(TEntry::comparator, &(*m_array).m_sortContext);
	int n = (*m_array).getOrder(e);
	[tvEntry selectRow:n byExtendingSelection:NO];
	[tvEntry reloadData];
	[tvEntry scrollRowToVisible:n];
*/

}

- (void)sheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo {
	if (m_newEntry != NULL) {
		if (returnCode == 1) {
			(*m_array).add(m_newEntry);
			(*m_array).sortValues(TEntryMap::comparator, &(*m_array).m_sortContext);
			int n = (*m_array).getOrder(m_newEntry);
			[tvEntry selectRow:n byExtendingSelection:NO];
			[tvEntry reloadData];
			[tvEntry scrollRowToVisible:n];
		}
		else {
			delete m_newEntry;
		}
		m_newEntry = NULL;
	}
	[self fixControl];
}

- (IBAction)menuSaveAction:(id)sender {
	int n = [tvEntry selectedRow];
	if ((*m_array).saveMapEntry() == false)
		return;
 	(*m_array).sortValues(TEntryMap::comparator, &(*m_array).m_sortContext);
	[tvEntry reloadData];
	[tvEntry selectRow:n byExtendingSelection:NO];
	[tvEntry scrollRowToVisible:n];
	[self fixControl];
}

- (IBAction)menuRevertAction:(id)sender {
	(*m_array).detachIC();
	if ((*m_array).attachIC() == false) {
		// Alert
		[[NSApplication sharedApplication] terminate:self];
	}
	(*m_array).createEntryArray();
	(*m_array).sortValues(TEntryMap::comparator, &(*m_array).m_sortContext);
	[tvEntry reloadData];
	[self fixControl];
}

- (IBAction)menuDeleteAction:(id)sender {
	int n = [tvEntry selectedRow];
	if (n < 0)
		return;
	(*m_array).onDelete(n);
	if (n >= (*m_array).getCount())
		n = (*m_array).getCount() - 1;
 	(*m_array).sortValues(TEntryMap::comparator, &(*m_array).m_sortContext);
	[tvEntry reloadData];
	if (n >= 0) {
		[tvEntry selectRow:n byExtendingSelection:NO];
		[tvEntry scrollRowToVisible:n];
	}
	[self fixControl];
}

- (IBAction)doubleClickTvEntryAction:()sender {
	[self menuModifyAction:sender];
}

- (IBAction)btnDeleteAction:(id)sender
{
	[self menuDeleteAction:sender];
}

- (IBAction)btnModifyAction:(id)sender
{
	[self menuModifyAction:sender];
}

- (IBAction)btnNewAction:(id)sender
{
	[self menuNewAction:sender];
}

- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication *)sender {
	[self ending];
	return NSTerminateNow;
}

- (void)windowWillClose:(NSNotification *)notif {
	[[NSApplication sharedApplication] terminate:self];
}

- (void)ending {
	if (m_array == NULL)
		return;

	[self saveSetting];

	if ((*m_array).hasDirty()) {
		NSAlert *alert = [NSAlert alertWithMessageText:@"Not Saved." 
			defaultButton:@"Save" alternateButton:@"Discard" otherButton:@""
			informativeTextWithFormat:@"Do You Save Internet Config Extension Map?"];
		int select = [alert runModal];
		if (select == 1) {
			(*m_array).saveMapEntry();
		}
	}

	for (int i = 0; i < (*m_array).getCount(); i++) {
		delete (TEntry *)(*m_array)[i];
	}
	delete m_array;
	m_array = NULL;
}


@end
