#import "TMappingDlg.h"
#import "CClass.h"

@implementation TMappingDlg

- (id)init {
	[super init];
	m_result = NO;
	m_array = NULL;
	m_entry = NULL;
	m_entrySaved = NULL;
	m_applPath = new CString;
	m_applInfo = new TApplInfo;
	return self;
}

- (void)dealloc {
	if (m_applPath != NULL)
		delete m_applPath;
	if (m_applInfo != NULL)
		delete m_applInfo;
	[super dealloc];
}

- (NSWindow *)window {
	return window;
}

- (void)setup:(TEntry *)entry andArray:(TEntryMap *)entryArray {
	[window setDelegate: self];
	m_entry = entry;
	m_array = entryArray;
	m_entrySaved = (*m_entry).clone();
	[tfEntryName setStringValue:(*m_entry).m_entryName];
	[tfCreator setStringValue:(*m_entry).m_fileCreator];
	[cbxFileType setStringValue:(*m_entry).m_fileType];
	[tfExtension setStringValue:(*m_entry).m_extension];
	[tfApplication setStringValue:(*m_entry).m_creatorAppName];
	[tfMIMEType setStringValue:(*m_entry).m_mimeType];
	[tfPostCreator setStringValue:(*m_entry).m_postCreator];
	[tfPostApplication setStringValue:(*m_entry).m_postAppName];

	int flags = (*m_entry).m_data.flags;
	[chkBinary setIntValue:(flags & kICMapBinaryMask) ? 1 : 0];
	[chkResourceFork setIntValue:(flags & kICMapResourceForkMask) ? 1 : 0];
	[chkDataFork setIntValue:(flags & kICMapDataForkMask) ? 1 : 0];
	[chkPostProcess setIntValue:(flags & kICMapPostMask) ? 1 : 0];
	[chkNotIncoming setIntValue:(flags & kICMapNotIncomingMask) ? 1 : 0];
	[chkNotOutgoing setIntValue:(flags & kICMapNotOutgoingMask) ? 1 : 0];

	*m_applPath = "";
	if ((*m_entry).m_creatorAppName.getLength() > 0) {
		if ((*m_entry).m_creatorAppName.hasSuffix(".app") == false)
			*m_applPath = [[NSWorkspace sharedWorkspace] fullPathForApplication:(*m_entry).m_creatorAppName + ".app"];
		if ((*m_applPath).getLength() == 0)
			*m_applPath = [[NSWorkspace sharedWorkspace] fullPathForApplication:(*m_entry).m_creatorAppName];
	}

	(*m_applInfo).clear();
	if ((*m_applPath).getLength() > 0) {
		(*m_applInfo).load(*m_applPath);
	}

	[cbxFileType setUsesDataSource:YES];
	[cbxFileType setDataSource:self];
}

/*
- (BOOL)doModal {
	m_result = NO;
	[NSApp runModalForWindow:window];
	return m_result;
}
*/

- (void)doSheet:(NSWindow *)parent modalDelegate:(id)modalDelegate didEndSelector:(SEL)didEndSelector contextInfo:(void *)contextInfo {
	[NSApp beginSheet:window modalForWindow:parent modalDelegate:modalDelegate didEndSelector:didEndSelector contextInfo:contextInfo];
}

- (int)numberOfItemsInComboBox:(NSComboBox *)cbx {
	if (cbx == cbxFileType) {
		return (*m_applInfo).m_fdTypes.getCount();
	}
	return 0;
}

- (id)comboBox:(NSComboBox *)cbx objectValueForItemAtIndex:(int)index {
	if (cbx == cbxFileType) {
		if (index < 0 || index >= (*m_applInfo).m_fdTypes.getCount())
			return nil;
		CString *s = (*m_applInfo).m_fdTypes[index];
		return *s;
	}
	return nil;
}

//

- (IBAction)btnOKAction:(id)sender {
	CString extension([tfExtension stringValue]);
	CString mimeType([tfMIMEType stringValue]);
	if (extension.getLength() == 0 && mimeType.getLength() == 0) {
		// ALERT
		NSAlert *alert = [NSAlert alertWithMessageText:@"Input Error." 
			defaultButton:@"OK" alternateButton:@"" otherButton:@""
			informativeTextWithFormat:@"Input Extension or MIME type."];
		[alert runModal];
		return;
	}
	CString creator([tfCreator stringValue]);
	CString fileType([cbxFileType stringValue]);
	if ((*m_array).serarchResemble(extension, mimeType, creator, fileType, m_entry) != NULL) {
		// ALERT
		NSAlert *alert = [NSAlert alertWithMessageText:@"Input Error." 
			defaultButton:@"OK" alternateButton:@"" otherButton:@""
			informativeTextWithFormat:@"Same Combination Exists."];
		[alert runModal];
		return;
	}
	(*m_entry).m_entryName = [tfEntryName stringValue];
	(*m_entry).m_fileCreator = [tfCreator stringValue];
	(*m_entry).m_fileType = [cbxFileType stringValue];
	(*m_entry).m_extension = [tfExtension stringValue];
	(*m_entry).m_creatorAppName = [tfApplication stringValue];
	(*m_entry).m_mimeType = [tfMIMEType stringValue];
	(*m_entry).m_postCreator = [tfPostCreator stringValue];
	(*m_entry).m_postAppName = [tfPostApplication stringValue];

	int flags = 0;
	flags |= [chkBinary intValue] ? kICMapBinaryMask : 0;
	flags |= [chkResourceFork intValue] ? kICMapResourceForkMask : 0;
	flags |= [chkDataFork intValue] ? kICMapDataForkMask : 0;
	flags |= [chkPostProcess intValue] ? kICMapPostMask : 0;
	flags |= [chkNotIncoming intValue] ? kICMapNotIncomingMask : 0;
	flags |= [chkNotOutgoing intValue] ? kICMapNotOutgoingMask : 0;
	(*m_entry).m_data.flags = flags;

	if ((*m_entry) != (*m_entrySaved))
		(*m_entry).m_dirty = true;
	m_result = YES;
	[window close];
}

- (IBAction)btnCancelAction:(id)sender
{
	m_result = NO;
	[window close];
}

- (IBAction)btnSelectAction:(id)sender
{
	NSFileManager *fm = [NSFileManager defaultManager];
	NSOpenPanel *dlg = [NSOpenPanel openPanel];
	NSString *directory = [(NSString *)*m_applPath stringByDeletingLastPathComponent];
	NSString *appName = [(NSString *)*m_applPath lastPathComponent];
	if ([dlg runModalForDirectory:directory file:appName types:nil] == NSOKButton) {
		*m_applPath = [dlg filename];
		[tfApplication setStringValue:[fm displayNameAtPath:*m_applPath]];
		(*m_applInfo).load((CFStringRef)[dlg filename]);
		[tfCreator setStringValue:(*m_applInfo).m_creator];
		[cbxFileType reloadData];
	}
}

- (void)windowWillClose:(NSNotification *)notif {
	delete m_entrySaved;
	m_entrySaved = NULL;

	[window orderOut:self];
	[NSApp endSheet:window returnCode:m_result ? 1 : 0];
//	[NSApp stopModal];
}

@end
